#include <iostream>
#include <vector>
#include <bitset>
#include <algorithm>
#include <stdexcept>
#include <memory.h>
#include <string>
#include <sstream>
#include <cassert>
#include <functional>
using namespace std;

typedef bitset<8> bs8;

const int FIELD_WIDTH = 20;
const int FIELD_HEIGHT = 14;

struct TPoint {
    int X, Y;
    TPoint()
        : X(-1), Y(-1)
    {}
    TPoint(int x, int y)
        : X(x), Y(y)
    {}

    inline bool operator==(const TPoint& p) const {
        return (X == p.X && Y == p.Y);
    }
    inline bool operator!=(const TPoint& p) const {
        return !(*this == p);
    }
};

inline bool IsNotOutside(int x, int y) {
    return x >= 0 && x <= FIELD_WIDTH && y >= 0 && y <= FIELD_HEIGHT;
}

inline bool IsNotOutside(const TPoint& p) {
    return IsNotOutside(p.X, p.Y);
}

inline bool IsOnFieldBorder(int x, int y) {
    return x == 0 || x == FIELD_WIDTH || y == 0 || y == FIELD_HEIGHT;
}

inline bool IsOnFieldBorder(const TPoint& p) {
    return IsOnFieldBorder(p.X, p.Y);
}

bool HadStartCommand = false;

const TPoint* GetAllAdjacents(int x, int y) {
    struct TAdjacentHolder {
        TPoint Adjacents[FIELD_WIDTH + 1][FIELD_HEIGHT + 1][8];
        TAdjacentHolder() {
            for (int i = 0; i <= FIELD_WIDTH; ++i)
                for (int j = 0; j <= FIELD_HEIGHT; ++j) {
                    TPoint* cur = Adjacents[i][j];
                    cur[0] = TPoint(i - 1, j - 1);
                    cur[1] = TPoint(i - 1, j);
                    cur[2] = TPoint(i - 1, j + 1);
                    cur[3] = TPoint(i, j - 1);
                    cur[4] = TPoint(i, j + 1);
                    cur[5] = TPoint(i + 1, j - 1);
                    cur[6] = TPoint(i + 1, j);
                    cur[7] = TPoint(i + 1, j + 1);
                }
        }
    };
    static TAdjacentHolder adjacentHolder;
    assert(IsNotOutside(x, y));
    return adjacentHolder.Adjacents[x][y];
}

const TPoint* GetAllAdjacents(const TPoint& p) {
    return GetAllAdjacents(p.X, p.Y);
}

typedef vector<TPoint> TMove;

class TPlayField {
public:
    bs8 UsedEdges[FIELD_WIDTH + 1][FIELD_HEIGHT + 1];
    TPoint CurrentPosition;

    TPlayField() {
        CurrentPosition = TPoint(FIELD_WIDTH / 2, FIELD_HEIGHT / 2);
        for (int i = 0; i <= FIELD_WIDTH; ++i) {
            for (int j = 0; j <= FIELD_HEIGHT; ++j) {
                const TPoint* adj = GetAllAdjacents(i, j);
                for (int k = 0; k < 8; k++) {
                    if ( !IsNotOutside(adj[k]) 
                        ||
                        (IsOnFieldBorder(i, j) && IsOnFieldBorder(adj[k]) && abs(i - adj[k].X) + abs(j - adj[k].Y) == 1))
                    {
                        UsedEdges[i][j][k] = true;
                    }
                }
            }
        }
    }

    TPlayField(const TPlayField& p) {
        CurrentPosition = p.CurrentPosition;
        for (int i = 0; i <= FIELD_WIDTH; ++i)
            for (int j = 0; j <= FIELD_HEIGHT; ++j)
                UsedEdges[i][j] = p.UsedEdges[i][j];
    }

    template<typename Func> void IterateEdges(int x, int y, Func f) const {
        const TPoint* allAdj = GetAllAdjacents(x, y);
        for (int i = 0; i < 8; ++i) {
            if (!UsedEdges[x][y][i])
                f(allAdj[i]);
        }
    }

    inline static bool IsGoal(int x, int y) {
        if (x != 0 && x != FIELD_WIDTH)
            return false;
        return (y > 5 && y < 9);
    }

    inline bool IsUsedCell(int x, int y) const {
        return UsedEdges[x][y].any() && !IsGoal(x, y);
    }

    inline static int GetId(int x, int y) {
        return x * (FIELD_HEIGHT + 1) + y;
    }
    inline static int GetId(const TPoint& p) {
        return GetId(p.X, p.Y);
    }

    inline static TPoint GetPointById(int id) {
        return TPoint(id / (FIELD_HEIGHT + 1), id % (FIELD_HEIGHT + 1));
    }

    void ApplyMove(const TMove& move) {
        assert(move.front() == CurrentPosition);
        for (TMove::const_iterator it = move.begin() + 1; it != move.end(); ++it) {
            const TPoint* possibleMoves = GetAllAdjacents(CurrentPosition);
            size_t ind = find(possibleMoves, possibleMoves + 8, *it) - possibleMoves;
            assert(ind <= 8);
            assert(!UsedEdges[CurrentPosition.X][CurrentPosition.Y][ind]);
            if (it + 1 == move.end())
                assert(!IsUsedCell(it->X, it->Y));
            else
                assert(IsUsedCell(it->X, it->Y));
            UsedEdges[CurrentPosition.X][CurrentPosition.Y][ind] = true;
            possibleMoves = GetAllAdjacents(*it);
            UsedEdges[it->X][it->Y][find(possibleMoves, possibleMoves + 8, CurrentPosition) - possibleMoves] = true;
            CurrentPosition = *it;
        }
    }
};

string ReadLine() {
    string result;
    getline(cin, result);
    return result;
}

TMove ParseMove(const string& str) {
    istringstream iss(str);
    int n;
    iss >> n;
    TMove result(n);
    for (int i = 0; i < n; i++) {
        iss >> result[i].Y >> result[i].X;
    }
    return result;
}

struct TMoveSearcher {
    const TPlayField& PlayField;
    TPoint Achievable[FIELD_WIDTH+1][FIELD_HEIGHT+1];
    TPoint BestCandidate;
    int BestWeight;
    int Level;
    TMoveSearcher(const TPlayField& playField, int level = 0)
        : PlayField(playField)
    {
        Level = level;
        for (int i = 0 ; i <= FIELD_WIDTH; ++i)
            for (int j = 0; j <= FIELD_HEIGHT; ++j) {
                Achievable[i][j] = TPoint(-1, -1);
            }
        BestCandidate = TPoint(-1, -1);
        BestWeight = -20000000;
    }
    int GetWeight(int x, int y) {
        if (PlayField.IsGoal(x, y)) {
            if ((HadStartCommand == (Level == 0)) != (x == 0))
                return 10000000;
            else
                return -10000000;
        }
        
        int result = x * 1000;
        int dToMid = abs(y - (FIELD_HEIGHT / 2));
        if (x < FIELD_HEIGHT / 2) {
            result += dToMid;
        } else
            result -= dToMid;
        if ((Level == 1) == HadStartCommand)
            result = -result;

        if (Level == 0) {
            TMove thisMove = GetMove(x, y);
            TPlayField f(PlayField);
            f.ApplyMove(thisMove);
            TMoveSearcher s(f, 1);
            s.DFS(f.CurrentPosition.X, f.CurrentPosition.Y, -1, -1);
            result -= s.BestWeight;
        }
        return result;
    }
    void UpdateBest(int x, int y) {
        int w = GetWeight(x, y);
        if (w > BestWeight) {
            BestCandidate = TPoint(x, y);
            BestWeight = w;
            return;
        }
    }
    void DFS(int x, int y, int prevX, int prevY) {
        if (Achievable[x][y] != TPoint(-1, -1)) {
            return;
        }
        Achievable[x][y] = TPoint(prevX, prevY);
        if (TPoint(x, y) != PlayField.CurrentPosition && !PlayField.IsUsedCell(x, y)) {
            UpdateBest(x, y);
        }
        if (TPoint(x, y) == PlayField.CurrentPosition || PlayField.IsUsedCell(x, y))
            PlayField.IterateEdges(x, y, TDFSBinder(this, x, y));
    }
    TMove GetMove(int x, int y) const {
        TMove result;
        TPoint cur(x, y);
        if (!IsNotOutside(cur))
            throw std::runtime_error("Can't make a turn");
        while (cur != PlayField.CurrentPosition) {
            result.push_back(cur);
            cur = Achievable[cur.X][cur.Y];
        }
        result.push_back(cur);
        reverse(result.begin(), result.end());
        return result;
    }
    struct TDFSBinder {
        TMoveSearcher* Ptr;
        int X, Y;
        TDFSBinder(TMoveSearcher* ptr, int x, int y)
            : Ptr(ptr)
            , X(x)
            , Y(y)
        {}
        void operator()(const TPoint& p) {
            Ptr->DFS(p.X, p.Y, X, Y);
        }
    };
};


TMove GetMyMove(const TPlayField &mainPlayField) {
    TMoveSearcher searcher(mainPlayField);
    searcher.DFS(mainPlayField.CurrentPosition.X, mainPlayField.CurrentPosition.Y, -57, -57);
    return searcher.GetMove(searcher.BestCandidate.X, searcher.BestCandidate.Y);
}

int main(void) {
    TPlayField mainPlayField;
    while (true) {
        string request = ReadLine();
        if (request == "Name") {
            cout << "SuckMyBalls" << endl;
        } else if (request == "Quit") {
            return 0;
        } else {
            if (request != "Start") {
                TMove move = ParseMove(request);
                mainPlayField.ApplyMove(move);
            } else {
                HadStartCommand = true;
            }
            TMove myMove = GetMyMove(mainPlayField);
            mainPlayField.ApplyMove(myMove);
            cout << myMove.size();
            for (TMove::const_iterator it = myMove.begin(); it != myMove.end(); ++it) {
                cout << " " << it->Y << " " << it->X;
            }
            cout << endl;
        }
        flush(cout);
    }
    return 0;
}
